//
//  M0motorPWM.cpp
//
//  **** FOR SAMD21 based boards only *****
//
// A simple PWM generator for controlling the speed of two motors independently.
//
// Configures SAMD21 based Arduino boards for 8 bit PWM at 23.45khz on analog pins A1 & A2 for motor control.
// Uses Generic clock 4 and TC4.
//
//  Created by Dale Heatherington on 5/3/16.
//  Version 0.2
//

/*This Code is licensed under a Creative Commons Attribution-ShareAlike 3.0 Unported License.
 *  This source code is provided as is, without warranty.
 *  You may copy and distribute verbatim copies of this document.
 *  You may modify and use this source code to create binary code
 *  for your own purposes, free or commercial.
 */



#include "M0motorPWM.h"
#include <Arduino.h>

M0motorPWM::M0motorPWM(void)
{
    per = 255;
}


//Default to non-inverted output
void M0motorPWM::start(int div) {
    
    M0motorPWM::start(div, false);
}

//Default to divisor = 8 and non-inverted output
void M0motorPWM::start(void) {
    
    M0motorPWM::start(8, false);
}



//------------------------------------------------------------------------
void M0motorPWM::start(int div, bool inv)         //Default divisor is 8 for 6 mhz clock
{

REG_GCLK_GENDIV = GCLK_GENDIV_DIV(div) |        // Divide the 48MHz clock source by divisor 8: 48MHz/8=6MHz
GCLK_GENDIV_ID(4);                              // Select Generic Clock (GCLK) 4
while (GCLK->STATUS.bit.SYNCBUSY);              // Wait for synchronization

REG_GCLK_GENCTRL = GCLK_GENCTRL_IDC |           // Set the duty cycle to 50/50 HIGH/LOW
GCLK_GENCTRL_GENEN |                            // Enable GCLK4
GCLK_GENCTRL_SRC_DFLL48M |                      // Set the 48MHz clock source
GCLK_GENCTRL_ID(4);                             // Select GCLK4
while (GCLK->STATUS.bit.SYNCBUSY);              // Wait for synchronization

// Enable the port multiplexer for the Arduino analog pins A1 and A2
PORT->Group[g_APinDescription[A1].ulPort].PINCFG[g_APinDescription[A1].ulPin].bit.PMUXEN = 1;
PORT->Group[g_APinDescription[A2].ulPort].PINCFG[g_APinDescription[A2].ulPin].bit.PMUXEN = 1;
    
//Enable waveform output on pins A1 & A2
PORT->Group[g_APinDescription[A1].ulPort].PMUX[g_APinDescription[A1].ulPin >> 1].reg =  PORT_PMUX_PMUXO_E |
                                                                                        PORT_PMUX_PMUXE_E;

// Feed GCLK4 to TC4 & TC5
REG_GCLK_CLKCTRL = GCLK_CLKCTRL_CLKEN |         // Enable GCLK4
GCLK_CLKCTRL_GEN_GCLK4 |                        // Select GCLK4
GCLK_CLKCTRL_ID_TC4_TC5;
while (GCLK->STATUS.bit.SYNCBUSY);              // Wait for synchronization

//PWM opertion setup
REG_TC4_CTRLA = TC_CTRLA_WAVEGEN_NPWM |
TC_CTRLA_MODE_COUNT8  |
TC_CTRLA_ENABLE |
TC_CTRLA_RUNSTDBY;
while (TC4->COUNT8.STATUS.bit.SYNCBUSY);

REG_TC4_CTRLC = 0;
if(inv == true) REG_TC4_CTRLC = TC_CTRLC_INVEN0 | TC_CTRLC_INVEN1;
while (TC4->COUNT8.STATUS.bit.SYNCBUSY);
    
    
REG_TC4_COUNT8_PER = per;
while (TC4->COUNT8.STATUS.bit.SYNCBUSY);

// Set the initial PWM signal duty cycle at 50%
REG_TC4_COUNT8_CC0 = 127;
while (TC4->COUNT8.STATUS.bit.SYNCBUSY);

REG_TC4_COUNT8_CC1 = 127;
while (TC4->COUNT8.STATUS.bit.SYNCBUSY);

}


//--------------------------------------------------------------

//Stop PWM and set pins high or low depending on x=HIGH or x=LOW
void M0motorPWM::stop(int x){
    
    pinMode(A1,OUTPUT);
    pinMode(A2,OUTPUT);
    
    digitalWrite(A1,x) ;
    digitalWrite(A2,x);
    
}


//--------------------------------------------------------------

//Sets PWM pulse width.  Range 0..255  cc0 is on A1,  cc1 is on pin A2

void M0motorPWM::setSpeed(int cc0, int cc1){
    
    REG_TC4_COUNT8_CC0 = cc0 & 255;
    while (TC4->COUNT8.STATUS.bit.SYNCBUSY);
    
    REG_TC4_COUNT8_CC1 = cc1 & 255;
    while (TC4->COUNT8.STATUS.bit.SYNCBUSY);
}


//------------------------------------------------------------------

void M0motorPWM::setPeriod(int n){
    
    per = n & 255;
    REG_TC4_COUNT8_PER = per;
    while (TC4->COUNT8.STATUS.bit.SYNCBUSY);
    
}


